@@ -28,7 +28,7 @@ def load_track(track, audio_channels, samplerate):
             samplerate=samplerate,
             channels=audio_channels)
     except FileNotFoundError:
-        errors['ffmpeg'] = 'Ffmpeg is not installed.'
+        errors['ffmpeg'] = 'FFmpeg is not installed.'
     except subprocess.CalledProcessError:
         errors['ffmpeg'] = 'FFmpeg could not read the file.'
 
@@ -61,6 +61,13 @@ def main():
                         default=Path("separated"),
                         help="Folder where to put extracted tracks. A subfolder "
                         "with the model name will be created.")
+    parser.add_argument("--filename",
+                        default="{track}/{stem}.{ext}",
+                        help="Set the name of output file. \n"
+                        'Use "{track}", "{trackext}", "{stem}", "{ext}" to use '
+                        "variables of track name without extension, track extension, "
+                        "stem name and default output file extension. \n"
+                        'Default is "{track}/{stem}.{ext}".')
     parser.add_argument("-d",
                         "--device",
                         default="cuda" if th.cuda.is_available() else "cpu",
@@ -75,11 +82,16 @@ def main():
                         default=0.25,
                         type=float,
                         help="Overlap between the splits.")
-    parser.add_argument("--no-split",
-                        action="store_false",
-                        dest="split",
-                        default=True,
-                        help="Doesn't split audio in chunks. This can use large amounts of memory.")
+    split_group = parser.add_mutually_exclusive_group()
+    split_group.add_argument("--no-split",
+                             action="store_false",
+                             dest="split",
+                             default=True,
+                             help="Doesn't split audio in chunks. "
+                             "This can use large amounts of memory.")
+    split_group.add_argument("--segment", type=int,
+                             help="Set split size of each chunk. "
+                             "This can help save memory of graphic card. ")
     parser.add_argument("--two-stems",
                         dest="stem", metavar="STEM",
                         help="Only separate audio into {STEM} and no_{STEM}. ")
@@ -110,9 +122,22 @@ def main():
     except ModelLoadingError as error:
         fatal(error.args[0])
 
+    if args.segment is not None and args.segment < 8:
+        fatal("Segment must greater than 8. ")
+
+    if '..' in args.filename.replace("\\", "/").split("/"):
+        fatal('".." must not appear in filename. ')
+
     if isinstance(model, BagOfModels):
         print(f"Selected model is a bag of {len(model.models)} models. "
               "You will see that many progress bars per track.")
+        if args.segment is not None:
+            for sub in model.models:
+                sub.segment = args.segment
+    else:
+        if args.segment is not None:
+            model.segment = args.segment
+
     model.cpu()
     model.eval()
 
@@ -140,12 +165,10 @@ def main():
                               num_workers=args.jobs)[0]
         sources = sources * ref.std() + ref.mean()
 
-        track_folder = out / track.name.rsplit(".", 1)[0]
-        track_folder.mkdir(exist_ok=True)
         if args.mp3:
-            ext = ".mp3"
+            ext = "mp3"
         else:
-            ext = ".wav"
+            ext = "wav"
         kwargs = {
             'samplerate': model.samplerate,
             'bitrate': args.mp3_bitrate,
@@ -155,18 +178,27 @@ def main():
         }
         if args.stem is None:
             for source, name in zip(sources, model.sources):
-                stem = str(track_folder / (name + ext))
-                save_audio(source, stem, **kwargs)
+                stem = out / args.filename.format(track=track.name.rsplit(".", 1)[0],
+                                                  trackext=track.name.rsplit(".", 1)[-1],
+                                                  stem=name, ext=ext)
+                stem.parent.mkdir(parents=True, exist_ok=True)
+                save_audio(source, str(stem), **kwargs)
         else:
             sources = list(sources)
-            stem = str(track_folder / (args.stem + ext))
-            save_audio(sources.pop(model.sources.index(args.stem)), stem, **kwargs)
+            stem = out / args.filename.format(track=track.name.rsplit(".", 1)[0],
+                                              trackext=track.name.rsplit(".", 1)[-1],
+                                              stem=args.stem, ext=ext)
+            stem.parent.mkdir(parents=True, exist_ok=True)
+            save_audio(sources.pop(model.sources.index(args.stem)), str(stem), **kwargs)
             # Warning : after poping the stem, selected stem is no longer in the list 'sources'
             other_stem = th.zeros_like(sources[0])
             for i in sources:
                 other_stem += i
-            stem = str(track_folder / ("no_" + args.stem + ext))
-            save_audio(other_stem, stem, **kwargs)
+            stem = out / args.filename.format(track=track.name.rsplit(".", 1)[0],
+                                              trackext=track.name.rsplit(".", 1)[-1],
+                                              stem="no_"+args.stem, ext=ext)
+            stem.parent.mkdir(parents=True, exist_ok=True)
+            save_audio(other_stem, str(stem), **kwargs)
 
 
 if __name__ == "__main__":
